/*--------------------------------------------------------------*/
/* xcircuit.i:							*/
/*	SWIG module generator for xcircuit			*/
/*--------------------------------------------------------------*/

%module xcwrap

%{

#include <tk.h>

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

#include "xcircuit.h"
#include "cursors.h"
#include "colordefs.h"
#include "menudep.h"
#include "prototypes.h"

Tcl_Interp *xcinterp;

extern Display *dpy;
extern Window win;
extern Colormap cmap;
extern Pixmap   STIPPLE[STIPPLES];  /* Polygon fill-style stipple patterns */
extern char _STR2[250];
extern char STIPDATA[STIPPLES][4];
extern Clientdata areastruct;

extern Tk_Window message1, message2, message3, wsymb, wschema, netbutton, corner;

#define STIPPLES 8

/*--------------------------------------*/
/* GUI Initialization under Tk		*/
/*--------------------------------------*/

void GUI_init(int argc, char *argv[])
{
   Tk_Window tkwind, tktop;
   int i;

   tktop = Tk_MainWindow(xcinterp);

   if (tktop == NULL) {
      fprintf(stderr, "No Top-Level Tk window available. . .\n");
      return;
   {

   /* Expect a top-level window created by the wish startup script */

   tkwind = Tk_NameToWindow(xcinterp, ".xcircuit", tktop);

   if (tkwind == NULL) {
      fprintf(stderr, "Expected the window hierarchy to be in place. . .\n");
      return;
   }

   /* Fill in global variables from the Tk window values */

   message1 = Tk_NameToWindow(xcinterp, ".xcircuit.menubar.message", tktop);
   message2 = Tk_NameToWindow(xcinterp, ".xcircuit.infobar.message1", tktop);
   message3 = Tk_NameToWindow(xcinterp, ".xcircuit.infobar.message2", tktop);
   areastruct.scrollbarv = Tk_NameToWindow(xcinterp,
	".xcircuit.mainframe.mainarea.sbleft", tktop);
   areastruct.scrollbarh = Tk_NameToWindow(xcinterp,
	".xcircuit.mainframe.mainarea.sbbottom", tktop);
   areastruct.area = Tk_NameToWindow(xcinterp,
	".xcircuit.mainframe.mainarea.drawing", tktop);
   corner = Tk_NameToWindow(xcinterp, ".xcircuit.mainframe.mainarea.corner", tktop);
   wsymb = Tk_NameToWindow(xcinterp, ".xcircuit.infobar.symb", tktop);
   wschema = Tk_NameToWindow(xcinterp, ".xcircuit.infobar.schem", tktop);
   netbutton = Tk_NameToWindow(xcinterp, ".xcircuit.menubar.netlistbutton", tktop);

   /* Setup event handlers for the drawing area and scrollbars			*/
   /* There are purposely no callback functions for these windows---they are	*/
   /* defined as type "frame" to keep down the cruft, as I will define my own	*/
   /* event handlers.								*/

   Tk_CreateEventHandler(areastruct.area, ExposureMask, drawarea, NULL);
   Tk_CreateEventHandler(areastruct.area, StructureNotifyMask, resizearea, NULL);
   Tk_CreateEventHandler(areastruct.area, ButtonPressMask | ButtonReleaseMask,
		buttonhandler, NULL);
   Tk_CreateEventHandler(areastruct.area, KeyPressMask | KeyReleaseMask,
		keyhandler, NULL);
   Tk_CreateEventHandler(areastruct.scrollbarh, ButtonMotionMask, panhbar, NULL);
   Tk_CreateEventHandler(areastruct.scrollbarv, ButtonMotionMask, panvbar, NULL);
   Tk_CreateEventHandler(areastruct.scrollbarh, StructureNotifyMask | ExposureMask,
		drawhbar, NULL);
   Tk_CreateEventHandler(areastruct.scrollbarv, StructureNotifyMask | ExposureMask,
		drawvbar, NULL);
   Tk_CreateEventHandler(areastruct.scrollbarh, ButtonReleaseMask, endhbar, NULL);
   Tk_CreateEventHandler(areastruct.scrollbarh, ButtonReleaseMask, endvbar, NULL);

   Tk_CreateEventHandler(corner, ButtonPressMask, zoomview, Number(1));
   Tk_CreateEventHandler(wsymb, ButtonPressMask, swapschem, Number(0));
   Tk_CreateEventHandler(wschema, ButtonPressMask, swapschem, Number(0));

   /* Build the pixmap images used by the menu buttons and toolbar */

   /* Build the toolbar */

   /* Make sure the window is mapped */

   Tk_Mapwindow(tkwind);

   dpy = Tk_Display(tkwind);
   win = Tk_WindowId(tkwind);
   cmap = Tk_Colormap(tkwind);

   /*-------------------------*/
   /* Create stipple patterns */
   /*-------------------------*/

   for (i = 0; i < STIPPLES; i++)
      STIPPLE[i] = XCreateBitmapFromData(dpy, win, STIPDATA[i], 4, 4);

   /*----------------------------------------*/
   /* Allocate space for the basic color map */
   /*----------------------------------------*/
   
   number_colors = 0; 
   colorlist = (colorindex *)malloc(sizeof(colorindex));
   appcolors = (int *) malloc(NUMBER_OF_COLORS * sizeof(int));

   /* (to be finished) */
}

/*--------------------------------------*/
/* Inline the main wrapper prodedure	*/
/*--------------------------------------*/

static int _xcircuit_start(ClientData clientData, Tcl_Interp *interp,
		int argc, char *argv[])
{
   /* Remember the interpreter */
   xcinterp = interp;

   fprintf(stderr, "Starting xcircuit under Tcl interpreter\n");

   pre_initialize();
   GUI_init(argc, argv);
   post_initialize();
   ghostinit();
   loadrcfile();
   composelib(PAGELIB);	/* make sure we have a valid page list */
   composelib(LIBLIB);	/* and library directory */
   if (argc == 2) {
      strcpy(_STR2, argv[1]);
      startloadfile();
   }
   else {
      findcrashfiles();
   }

   /* Return back to the interpreter; Tk is handling the GUI */
   return TCL_OK;
}

/*----------------------------------------------------------*/
/* Redefine all of the button callback functions for Tcl/Tk */
/*----------------------------------------------------------*/

/* Routines to implement: */
/* fontstyle, addtotext, addnewfont, */
/* fontencoding, sethjust, setvjust, setaction, boxedit, getgridtype, */
/* getdscale, newlibrary, startcatalog, newpagemenu, getwwidth, */
/* getline, getset, makebbox, addnewcolor, setcolor, startparam, */
/* getfill, getopaque, gettsize, startunparam, starttext, getosize, */
/* toggle,  selectfilter, startrotate, getgridspace, getsnapspace, */
/* dopintype, setpinvisible, dobeforeswap, startschemassoc, startconnect, */
/* callgennet, zoominrefresh, zoomoutrefresh, zoombox, centerpan, refresh, */
/* getfile, dooutput, getlib, getuserlib, savelib, resetbutton, quitcheck, */
/* inversecolor, doxschema, dotoolbar, getwirewidth, starthelp, startdelete, */
/* xc_undelete, startcopy, startedit, startdesel, startpush, popobject, */
/* selectsave, startarc, startbox, startspline, join */

{
%}

%init %{
{

   /* Create the commands */

   Tcl_CreateCommand(interp, "xcircuit::start", _xcircuit_start,
                        (ClientData)NULL, (Tcl_CmdDeleteProc *) NULL);

   /* Export the namespace commands */

   Tcl_Eval(interp, "namespace eval xcircuit namespace export *");
}
%}

